%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function fittingOutput = checkFittingonRealPatchAllScanlines(patch_scandata_all, ...
                                                             patch_scanline_all, ...
                                                             gamma_range, ...
                                                             min_prom_minima, ...
                                                             filter_linear_values, ...
                                                             hist_bins)
                                                         
if nargin<4
    min_prom_minima = 0;
end
if nargin<5
    filter_linear_values = 0;
end
if nargin<6
    hist_bins = gamma_range(1):0.1:gamma_range(end);
end
% For all the scalines in the patch
[dims, patchSize]  = size(patch_scandata_all{1});
if(dims ~= 3)
    error('Expected dims of patch in 3xpatchSize!\n');
end
distss_all = cell(1, patchSize);
distss2_all= cell(1, patchSize);
gamma_pred_distss_all = zeros(patchSize, 1);
gamma_pred_distss2_all= zeros(patchSize, 1);
minima_distss_all = zeros(patchSize, 1);
minima_distss2_all= zeros(patchSize, 1);
for n_line=1:patchSize
    % Get patch_scandata 
    patch_scandata = patch_scandata_all{n_line};
    patch_scanline  = patch_scanline_all{n_line};
    % Check fitting on the patch for various values of gamma
    distss = zeros(1, numel(gamma_range));
    distss2= zeros(1, numel(gamma_range));
    datatocheck = patch_scandata;
    for i=1:numel(gamma_range)
        coeffs         = gamma_range(i);
        datatochecktrn = crf_ggcm(datatocheck, coeffs);
        %datatochecktrn_scaled = scaleDatain0to1(datatochecktrn);
        datatochecktrn_scaled = meanNormalizeData(datatochecktrn);
        % 3D line fitting on the linearized/transformed points
        [~, dists] = fitline3d(datatochecktrn);
        distss(1, i) = mean(dists);
        [~, dists2] = fitline3d(datatochecktrn_scaled);
        distss2(1, i) = mean(dists2);
    end
    distss_all{n_line} = distss;
    distss2_all{n_line}= distss2;

    % Gamma predicted from distss (unscaled linearized distributions)
    [gamma_pred_distss, minima_distss] = predGammaFromDistssMinima(gamma_range, distss, ...
                                                                   min_prom_minima);
    %fprintf('patch_size=%d, n_line=%d, gamma_pred_distss=%f\n', ...
    %        patchSize, n_line, gamma_pred_distss);
    gamma_pred_distss_all(n_line) = gamma_pred_distss;
    minima_distss_all(n_line)     = minima_distss;

    % Gamma predicted from distss (unscaled linearized distributions)
    [gamma_pred_distss2, minima_distss2] = predGammaFromDistssMinima(gamma_range, distss2, ...
                                                                     min_prom_minima);
    gamma_pred_distss2_all(n_line) = gamma_pred_distss2;
    minima_distss2_all(n_line)     = minima_distss2;
end

% Filter out linear CRF predictions for mean, mode and std computations (if
% needed)??
if filter_linear_values == 1
    gamma_pred_distss_all_filtered =  gamma_pred_distss_all(gamma_pred_distss_all<0.9);
    gamma_pred_distss2_all_filtered=  gamma_pred_distss2_all(gamma_pred_distss2_all<0.9);
else
    gamma_pred_distss_all_filtered = gamma_pred_distss_all;
    gamma_pred_distss2_all_filtered= gamma_pred_distss2_all;
end
% But, if all the values are linear?, then accept as is, and reject later
% in the final analysis??
if isempty(gamma_pred_distss_all_filtered)
    gamma_pred_distss_all_filtered = gamma_pred_distss_all;
end
if isempty(gamma_pred_distss2_all_filtered)
    gamma_pred_distss2_all_filtered= gamma_pred_distss2_all;
end

% Gamma prediction through statistics
fittingOutput.distss_all                    = distss_all;
fittingOutput.distss2_all                   = distss2_all;
fittingOutput.gamma_pred_distss_all         = gamma_pred_distss_all;
fittingOutput.gamma_pred_distss2_all        = gamma_pred_distss2_all;
fittingOutput.gamma_pred_distss_all_mean    = mean(gamma_pred_distss_all_filtered);
fittingOutput.gamma_pred_distss2_all_mean   = mean(gamma_pred_distss2_all_filtered);
if filter_linear_values == 0
    fittingOutput.gamma_pred_distss_all_std    = std(gamma_pred_distss_all_filtered);
    fittingOutput.gamma_pred_distss2_all_std   = std(gamma_pred_distss2_all_filtered);
else
    true_cam_I = linspace(0, 1, 100);
    temp_distss_predCurves  = zeros(numel(gamma_pred_distss_all_filtered), numel(true_cam_I));
    temp_distss2_predCurves = zeros(numel(gamma_pred_distss2_all_filtered), numel(true_cam_I));
    for n=1:numel(gamma_pred_distss2_all_filtered)
        temp_distss_predCurves(n, :)  = crf_ggcm(true_cam_I, gamma_pred_distss_all_filtered(n));
        temp_distss2_predCurves(n, :) = crf_ggcm(true_cam_I, gamma_pred_distss2_all_filtered(n));
    end    
    fittingOutput.gamma_pred_distss_all_std = mean(std(temp_distss_predCurves,1,1));
    fittingOutput.gamma_pred_distss2_all_std = mean(std(temp_distss2_predCurves,1,1));
end
[hist_distss_values, hist_distss_edges]   = histcounts(gamma_pred_distss_all_filtered, ...
                                                       hist_bins);
hist_edges_s = hist_distss_edges(hist_distss_values==max(hist_distss_values));
hist_edges_s = hist_edges_s(1);
hist_edges_e = hist_edges_s+hist_bins(2)-hist_bins(1);
fittingOutput.gamma_pred_distss_all_mode   = mean(gamma_pred_distss_all_filtered(gamma_pred_distss_all_filtered >= hist_edges_s & ...
                                                                                 gamma_pred_distss_all_filtered <= hist_edges_e));
                                                                            
[hist_distss2_values, hist_distss2_edges]   = histcounts(gamma_pred_distss2_all_filtered, ...
                                                         hist_bins);
hist_edges_s = hist_distss2_edges(hist_distss2_values==max(hist_distss2_values));
hist_edges_s = hist_edges_s(1);
hist_edges_e = hist_edges_s+hist_bins(2)-hist_bins(1);
fittingOutput.gamma_pred_distss2_all_mode   = mean(gamma_pred_distss2_all_filtered(gamma_pred_distss2_all_filtered >= hist_edges_s & ...
                                                                                   gamma_pred_distss2_all_filtered <= hist_edges_e));
% min_minima_distss2_all = nanmin(minima_distss2_all);
% fittingOutput.min_minima_distss2_all = min_minima_distss2_all;
% fittingOutput.gamma_pred_distss2_min_minima = gamma_pred_distss2_all(minima_distss2_all == min_minima_distss2_all);
end
