%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [patchCenterCoords , patchScanDirn] = getPatches(img, patchSize, ...
                                                          varThrUniform, varThrDist, ...
                                                          minValThres, maxValThres, patchOverlapRate)
                                                      
if nargin<5
    minValThres = 40/255;
end
if nargin<6
    maxValThres = 230/255;
end
if nargin<7
    patchOverlapRate = 2;
end

% Normalize the image in the [0,1] interval (wrt maximum value)
img = im2double(img);
img = img ./ repmat(max(max(img)), [size(img,1) size(img,2)]);

% Remove the patches at the borders of the image
patchMap = img;
borderWidth  = floor(patchSize./2);
patchHalfSize= floor(patchSize./2);
for ch=1:size(img,3)
    patchMap(1:borderWidth(1), :, ch) = 0; patchMap(end-borderWidth(1)+1:end, :, ch) = 0; 
    patchMap(:, 1:borderWidth(1), ch) = 0; patchMap(:, end-borderWidth(1)+1:end, ch) = 0;
end

% Find patches 
[h_img, w_img, ~]  = size(img);
[col_ids, row_ids] = meshgrid(2*patchSize:round(patchSize/patchOverlapRate):w_img-2*patchSize, ...
                              2*patchSize:round(patchSize/patchOverlapRate):h_img-2*patchSize);
row_ids = row_ids(:);
col_ids = col_ids(:);
numTotalPatches    = numel(row_ids);
patchCenterCoords  = cell(1, numTotalPatches);
patchScanDirn      = cell(1, numTotalPatches);
for num_patch = 1:numTotalPatches

    cur_row_id    = row_ids(num_patch);
    cur_col_id    = col_ids(num_patch);
    patchRowRange = cur_row_id-patchHalfSize:cur_row_id+patchHalfSize;
    patchColRange = cur_col_id-patchHalfSize:cur_col_id+patchHalfSize;

    sqColorPatch = patchMap(patchRowRange, patchColRange, :);
    colorPatch   = permute(sqColorPatch, [3 1 2]);

    % No channel in the patch should be absolutely 0
    if~isempty(find(sqColorPatch<1e-3, 1))
        continue;
    end
    % No channel in the patch should be absolutely 1
    if~isempty(find(sqColorPatch>0.99, 1))
        continue;
    end
    % The three channels in the patch should not be less than 40/255
    if~isempty(find(mean(sqColorPatch,3)<=minValThres, 1))
        continue;
    end
    % The three channels in the patch should not be more than 230/255
    if~isempty(find(mean(sqColorPatch,3)>=maxValThres, 1))
        continue;
    end

    % compute the variance of the patch
    varRegionColor= var(colorPatch(:, :), [], 2);

    % check for variance in the three channels
    if varRegionColor(1) < varThrUniform && ...
       varRegionColor(2) < varThrUniform && ...
       varRegionColor(3) < varThrUniform % The patch is quite uniform then!
        continue;
    end
    if varRegionColor(1) < varThrDist || ...
       varRegionColor(2) < varThrDist || ...
       varRegionColor(3) < varThrDist % The distribution is quite cluttered then!
       continue;
    end
    
    % Passing the above tests, get the scan direction with max. variance
    varScanLines = zeros(patchSize, 2);
    for idx=1:patchSize
        sqColorPatchLineVert = reshape(sqColorPatch(:, idx, :), [patchSize, 3]);
        sqColorPatchLineHorz = reshape(sqColorPatch(idx, :, :), [patchSize, 3]);
        varScanLines(idx, 1) = min(var(sqColorPatchLineVert));
        varScanLines(idx, 2) = min(var(sqColorPatchLineHorz));
    end
    meanVarScanLines = mean(varScanLines, 1);
    if (meanVarScanLines(1) > meanVarScanLines(2))
        patchScanDirn{1, num_patch} = 1; % Verical, Up-Down
    else
        patchScanDirn{1, num_patch} = 2; % Horizontal, Left-Right
    end

    % Save patch center coords
    curpatchCenterCoords = [cur_row_id cur_col_id]';
    patchCenterCoords{1, num_patch} = curpatchCenterCoords;

end
patchCenterCoords = patchCenterCoords(~cellfun('isempty',patchCenterCoords));
patchScanDirn = cell2mat(patchScanDirn(~cellfun('isempty',patchScanDirn)));
end
